/*
    Copyright (C) 1995-1998 AROS - The Amiga Research OS
    $Id: addsegment.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: Add a segment to the resident list.
    Lang: english
*/
#include "dos_intern.h"
#include <exec/memory.h>
#include <proto/exec.h>

/*****************************************************************************

    NAME */
#include <dos/dosextens.h>
#include <proto/dos.h>

	AROS_LH3(LONG, AddSegment,

/*  SYNOPSIS */
	AROS_LHA(CONST_STRPTR, name, D1),
	AROS_LHA(BPTR  , seg, D2),
	AROS_LHA(LONG  , type, D3),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 129, Dos)

/*  FUNCTION
	Adds a program segment to the system resident list. You can later
	use these segments to run programs.

	The name field should refer to a NULL terminated strings, which
	will be copied. The type field determines the type of resident
	program. Normal programs should have type >= 0, system segments
	should have type == CMD_SYSTEM.

	Note that all other values of type are reserved.

    INPUTS
	name		- Name of the segment. This is used by FindSegment().
	seg		- Segment to add.
	type		- What type of segment (initial use count).

    RESULT
	Segment will have been added to the DOS resident list.

	!= 0	success
	== 0	failure

    NOTES

    EXAMPLE

    BUGS
	Uses Forbid() based locking.

    SEE ALSO
	FindSegment(), RemSegment()

    INTERNALS

    HISTORY
	27-11-96    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

    struct Segment *sptr;
    int namelen;

    DEBUG_SEGMENT(dprintf("AddSegment: name =\"%s\", seglist = $%lx, type = %ld\n",
			  name, seg, type));

#ifdef AROS_DOS_COMPATIBLE
    if (!DOSBase->dl_Root->rn_Info)
    {
	return DOSFALSE;
    }
#endif /* AROS_DOS_COMPATIBLE */

#if !USE_EARLYDOS
    FAILFORTASKNP(return DOSFALSE;)
#endif

    /* Make sure type is valid */
    /*
    if (type < -1)
    {
	return DOSFALSE;
    }
    */

    namelen = strlen(name);
    sptr = dosAllocVec(sizeof(struct Segment) + namelen - 3,
		       MEMF_CLEAR | MEMF_PUBLIC);

    if (sptr)
    {
	sptr->seg_UC  = type;
	sptr->seg_Seg = seg;

#ifdef AROS_FAST_BSTR

	CopyMem((STRPTR) name, sptr->seg_Name, namelen);

#else /* AROS_FAST_BSTR */

	CopyMem((STRPTR) name, &sptr->seg_Name[1], namelen);
	*sptr->seg_Name = namelen;

#endif /* AROS_FAST_BSTR */

	/* Sigh, we just add the segment to the start of the list */
	Forbid();

#ifdef AROS_DOS_COMPATIBLE
	{
	    struct DosInfo *di = BADDR(DOSBase->dl_Root->rn_Info);
	    sptr->seg_Next = (BPTR) di->di_NetHand;
	    di->di_NetHand = (APTR) MKBADDR(sptr);
	}

#else /* AROS_DOS_COMPATIBLE */

	sptr->seg_Next = DOSBase->dl_ResList;
	DOSBase->dl_ResList = MKBADDR(sptr);

#endif /* AROS_DOS_COMPATIBLE */

	Permit();

	DEBUG_SEGMENT(dprintf("AddSegment: returned $%lx\n", MKBADDR(sptr)));

	return DOSTRUE;
    }

    DEBUG_SEGMENT(dprintf("AddSegment: failed\n"));

    return DOSFALSE;

    AROS_LIBFUNC_EXIT
} /* AddSegment */
